(function(){
	header.xmas = {};

	/**
	 * Header Factory
	 *
	 * Expects:
	 * object = {
	 *		label      : (string)  Center Label, if ommited displays logotype.
	 *      labelRight : (string)  Right Button Label, if ommited, hides button.
	 *      labelLeft  : (string)  Left  Button Label, if ommited, hides button.
	 * }
	 *
	 * Returns:
	 * object = {
	 *		view        : Ti.UI.view
	 *		label       : Ti.UI.label
	 *		buttonLeft  : Ti.UI.Button (if specified)
	 *		labelLeft   : Ti.UI.label  (if specified)
	 *		buttonRight : Ti.UI.Button (if specified)
	 *		labelRight  : Ti.UI.label  (if specified)
	 * }
	 *
	 * Note:
	 * If custom fonts are not rendering on iPhone, follow this tutorial:
	 * http://www.rafaelhernandez.org/custom_fonts.mov
	**/
	header.xmas.get = function(obj){

		var elements = {};

		if (typeof obj != 'object')  obj = {};
		if (typeof obj.label      != 'string') obj.label = false;
		if (typeof obj.labelRight != 'string') obj.labelRight = false;
		if (typeof obj.labelLeft  != 'string') obj.labelLeft  = false;

		elements.view =  Ti.UI.createView({
			top             : 0,
			left            : 0,
			width           : 320,
			height          : 56,
			backgroundImage : '/images/ordenarTitle.png',
			backgroundColor : 'transparent'
		});
		// -------------------------------------------------------- Center Label
		//                                                          Logo or Text
		if (obj.label) {
			elements.label = Ti.UI.createLabel({
				text            : obj.label.toUpperCase(),
				top             : 0,
				width           : 320,
				height          : 56,
				textAlign       : 'center',
				color           : '#FFF',
				backgroundColor :'transparent',
				font: {
					fontSize:20,
					fontFamily:'Impact',
					fontWeight:'bold'
				},
			});
		} else {
			elements.label = Ti.UI.createImageView({
				image  : '/images/headerLabel.png',
				width  : 98,
				height : 20,
				top    : 21,
				left   : 109
			})
		}
		elements.view.add(elements.label);

		// --------------------------------------------------------  Left Button

		if (obj.labelLeft){
			elements.buttonLeft = Ti.UI.createButton({
				top             : 7,
				left            : 1,
				width           : '101px',
				height          : '41px',
				backgroundImage : 'images/headerButtonLeft.png'
			});
			elements.labelLeft = Ti.UI.createLabel({
				top:-5,
				text            : obj.labelLeft,
				color           : '#FFF',
				textAlign       : 'center',
				backgroundColor :'transparent',
				font:{
					fontSize:14,
					fontWeight:'bold'
				},
			});
			elements.buttonLeft.add(elements.labelLeft);
			elements.view.add(elements.buttonLeft);
		}
		// -------------------------------------------------------- Right Button

		if (obj.labelRight){
			elements.buttonRight = Ti.UI.createButton({
				top    : 7,
				left   : 219,
				width  : '101px',
				height : '41px',
				backgroundImage:'images/headerButtonRight.png'
			});
			elements.labelRight = Ti.UI.createLabel({
				top            : -5,
				text           : obj.labelRight,
				textAlign      : 'center',
				color          : '#FFF',
				backgroundColor: 'transparent',
				font:{
					fontSize:14,
					//fontFamily:'Impact',
					fontWeight:'bold'
				}
			});
			elements.buttonRight.add(elements.labelRight);
			elements.view.add(elements.buttonRight);
		}
		return elements;
	}

})();