(function(){

	spinner.xmas = {};

	/**
	 * Expects:
	 *
	 * object = {
	 * 		window: Ti.UI.createWindow(), // window element     (required)
	 * 		properties:{                  // picker properties  (optional)
	 * 			top:50,
	 * 			selectionIndicator:true
	 * 		},
	 * 		data:{                        // picker elements (optional, but stupid if left empty)
	 * 			ano : {
	 * 				'1981' : 1981,
	 * 				'2000' : 2000
	 * 			},
	 * 			mes : {
	 * 				'ENE' : 1,
	 * 				'FEB' : 2
	 * 			},
	 * 			dia : {
	 * 				'01' : 1,
	 * 				'02' : 2,
	 * 				'03' : 3
	 * 			}
	 * 		},
	 * 		selected:{                    // selected values (optional)
	 * 			ano:'2000',
	 * 			mes:'FEB',
	 * 			dia:'02'
	 * 		},
	 * });
	 *
	 * Returns:
	 *
	 *  {
	 *	 	element : Ti.UI.picker,		  // picker element
	 *	 	data    : {					  // currently selected values (auto-update)
	 *	 		ano : 2000,
	 *	 		mes : 2,
	 *	 		dia : 1
	 *	 	}
	 *	}
	 */
	spinner.xmas.get = function(obj){

		if (typeof obj.window != 'object' || !obj.window.toString().indexOf('TiUIWindow'))
			return alert('Window element is needed due to a bug.');
		// data sanitization (yeah right)
		if (typeof obj            != 'object') obj            = { properties:{}, data:{} };
		if (typeof obj.properties != 'object') obj.properties = {};
		if (typeof obj.data       != 'object') obj.data       = {};
		if (typeof obj.selected   != 'object') obj.selected   = {};

		// this validation sucks, make it better
		if (typeof obj.button   != 'object') return alert('Object button is needed.')
		if (typeof obj.callback != 'function') obj.callback = function(){};

		
		/**
		 * Extends old object with new (returns a copy)
		 *
		 * @author   Hector Menendez <h@cun.mx>
		 * @created  2011/NOV/21 17:41
		 */
		var extend = function(old, neo){
			if (typeof old != 'object') old = {};
			if (typeof neo != 'object') neo = {};
			var result = {}, target, i;
			// don't rewrite, just copy
			for (property in old) result[property] = old[property];
			for (property in neo){
				// no need to parse if both objects are the same
				if (typeof neo[property] === result[property]) continue;
				result[property] =  neo[property]
			}
			return result;
		}
		

		/////////////////////////////////////////////////////////////// ELEMENTS



		var picker = Ti.UI.createPicker(extend({zIndex:9,selectionIndicator:true,top:245},obj.properties));
		var result = {};
		var c = 0, r, selected = [];
		var column = [];

		for (col in obj.data) {
			selected[c] = 0; // first value selected by default
			if  (typeof obj.data[col] != 'object') continue;
			data = [];
			temp = [];
			r = 0;
			// generate rows
			for (row in obj.data[col]){
				data.push(Ti.UI.createPickerRow({
					title : row.toString(),
					value : obj.data[col][row]
					// does not work because of known bug:
					// http://developer.appcelerator.com/question/54261/tiuipickersetselectedrow-not-working
					// using hacky workaround.
					// selected : (typeof obj.selected[col] != 'undefined' && obj.selected[col] == row)
				}));
				if (typeof obj.selected[col] != 'undefined' && obj.selected[col] == row)
					selected[c] = r;
				temp.push(row.toString().length+1);
				r++;
			}
			column[c] = {
				// get max string length for this column (array of rows);
				'max'  : Math.max.apply(Math, temp),
				// store  row elements for later creation
				'data' : data
			}
			c++;
		}
		var	i, percent, total=0, width=320;
		// calculate total width
		for (i in column) total += column[i].max;
		// generate columns with relative width to content
		i = 0;
		for (col in obj.data){
			picker.add(Ti.UI.createPickerColumn({
				title : col.toString(),
				rows  : column[i].data,
				width : width*(column[i].max/total)
			}));
			i++;
		}

		// Create support elements
		var viewMain = Ti.UI.createView({
			top             : 480,
			left            : 0,
			width           : 320,
			height          : 480,
			zIndex			: 9999
		});

		var viewGhost = Ti.UI.createView({
			top             : 0,
			height          : 210,
			backgroundColor : 'transparent'
		});

		var viewBar = Ti.UI.createView({
			top             : 207,
			width           : 320,
			height          : 50,
			borderRadius    : 5,
			backgroundColor : '#8F8F96',
			zIndex			: 2
		});

		var buttonBar = Ti.UI.createButtonBar({
		    labels          : ['Cancelar','Listo'],
		    top             : 215,
		    height          : 25,
		    width           : 298,
		    backgroundColor : '#333',
		    zIndex			: 3
		    //style           : Ti.UI.iPhone.SystemButtonStyle.BAR
		});

		viewMain.add(viewGhost);
		viewMain.add(viewBar);
		viewMain.add(buttonBar);
		viewMain.add(picker);

		///////////////////////////////////////////////////////////////// EVENTS

		var spinnerShow = function(){
			viewMain.animate({
				top      : 0,
				duration : 250
			});
		}

		var spinnerHide = function(){
			viewMain.animate({
				top      : 480,
				duration : 250
			});
		}

		obj.button.addEventListener('click',spinnerShow);

		viewGhost.addEventListener('click', spinnerHide);


		buttonBar.addEventListener('click',function(e){
			// index[0] = Cancelar, index[1] = Listo
			if (!e.index) return spinnerHide();
			obj.callback.call(result,e);
			spinnerHide();
		});
		
		obj.window.add(viewMain);

		// ugly workaround for  setting selected elements
		obj.window.addEventListener('open', function(){
			for (i in selected)	picker.setSelectedRow(i, selected[i], true);
		});
		// update values on change
		picker.addEventListener('change', function(e){
			for (col in obj.data){
				if (typeof obj.data[col] != 'object' || e.column.title != col) continue;
				result[col] = e.row.value;
			}
		});

		return {
			element : picker,
			data    : result,
			view    : viewMain
		};
	};

	////////////////////////////////////////////////////////// Reverse CONSTANTS

	spinner.xmas.reverse = function(target){
		var that = [];
		for(key in target) that[target[key]] = key;
		return that;
	}

	////////////////////////////////////////////////////////// SPECIAL CONSTANTS

	spinner.xmas.MONTH = {
		'Enero'      : 1,
		'Febrero'    : 2,
		'Marzo'      : 3,
		'Abril'      : 4,
		'Mayo'       : 5,
		'Junio'      : 6,
		'Julio'      : 7,
		'Agosto'     : 8,
		'Septiembre' : 9,
		'Octubre'    : 10,
		'Noviembre'  : 11,
		'Diciembre'  : 12
	};

	spinner.xmas.YEAR = (function(){
		var
			ano     = {},
			curr = new Date().getFullYear();
		for (var i = 1921; i <= curr; i++) ano[i] = i;
		return ano;
	})();

	spinner.xmas.DAY = (function(){
		var	dia = {};
		for (var i=1; i <= 31; i++) dia[i] = i;
		return dia;
	})();


	spinner.xmas.COUNTRY = {
		'México' : 1,
		'Afganistán' : 2,
		'Akrotiri' : 3,
		'Albania' : 4,
		'Alemania' : 5,
		'Andorra' : 6,
		'Angola' : 7,
		'Anguila' : 8,
		'Antártida' : 9,
		'Antigua y Barbuda' : 10,
		'Antillas Neerlandesas' : 11,
		'Arabia Saudí' : 12,
		'Arctic Ocean' : 13,
		'Argelia' : 14,
		'Argentina' : 15,
		'Armenia' : 16,
		'Aruba' : 17,
		'Ashmore and Cartier Islands' : 18,
		'Atlantic Ocean' : 19,
		'Australia' : 20,
		'Austria' : 21,
		'Azerbaiyán' : 22,
		'Bahamas' : 23,
		'Bahráin' : 24,
		'Bangladesh' : 25,
		'Barbados' : 26,
		'Bélgica' : 27,
		'Belice' : 28,
		'Benín' : 29,
		'Bermudas' : 30,
		'Bielorrusia' : 31,
		'Birmania; Myanmar' : 32,
		'Bolivia' : 33,
		'Bosnia y Hercegovina' : 34,
		'Botsuana' : 35,
		'Brasil' : 36,
		'Brunéi' : 37,
		'Bulgaria' : 38,
		'Burkina Faso' : 39,
		'Burundi' : 40,
		'Bután' : 41,
		'Cabo Verde' : 42,
		'Camboya' : 43,
		'Camerún' : 44,
		'Canadá' : 45,
		'Chad' : 46,
		'Chile' : 47,
		'China' : 48,
		'Chipre' : 49,
		'Clipperton Island' : 50,
		'Colombia' : 51,
		'Comoras' : 52,
		'Congo' : 53,
		'Coral Sea Islands' : 54,
		'Corea del Norte' : 55,
		'Corea del Sur' : 56,
		'Costa de Marfil' : 57,
		'Costa Rica' : 58,
		'Croacia' : 59,
		'Cuba' : 60,
		'Dhekelia' : 61,
		'Dinamarca' : 62,
		'Dominica' : 63,
		'Ecuador' : 64,
		'Egipto' : 65,
		'El Salvador' : 66,
		'El Vaticano' : 67,
		'Emiratos Árabes Unidos' : 68,
		'Eritrea' : 69,
		'Eslovaquia' : 70,
		'Eslovenia' : 71,
		'España' : 72,
		'Estados Unidos' : 73,
		'Estonia' : 74,
		'Etiopía' : 75,
		'Filipinas' : 76,
		'Finlandia' : 77,
		'Fiyi' : 78,
		'Francia' : 79,
		'Gabón' : 80,
		'Gambia' : 81,
		'Gaza Strip' : 82,
		'Georgia' : 83,
		'Ghana' : 84,
		'Gibraltar' : 85,
		'Granada' : 86,
		'Grecia' : 87,
		'Groenlandia' : 88,
		'Guam' : 89,
		'Guatemala' : 90,
		'Guernsey' : 91,
		'Guinea' : 92,
		'Guinea Ecuatorial' : 93,
		'Guinea-Bissau' : 94,
		'Guyana' : 95,
		'Haití' : 96,
		'Honduras' : 97,
		'Hong Kong' : 98,
		'Hungría' : 99,
		'India' : 100,
		'Indian Ocean' : 101,
		'Indonesia' : 102,
		'Irán' : 103,
		'Iraq' : 104,
		'Irlanda' : 105,
		'Isla Bouvet' : 106,
		'Isla Christmas' : 107,
		'Isla Norfolk' : 108,
		'Islandia' : 109,
		'Islas Caimán' : 110,
		'Islas Cocos' : 111,
		'Islas Cook' : 112,
		'Islas Feroe' : 113,
		'Islas Georgia del Sur y Sandwich del Sur' : 114,
		'Islas Heard y McDonald' : 115,
		'Islas Malvinas' : 116,
		'Islas Marianas del Norte' : 117,
		'Islas Marshall' : 118,
		'Islas Pitcairn' : 119,
		'Islas Salomón' : 120,
		'Islas Turcas y Caicos' : 121,
		'Islas Vírgenes Americanas' : 122,
		'Islas Vírgenes Británicas' : 123,
		'Israel' : 124,
		'Italia' : 125,
		'Jamaica' : 126,
		'Jan Mayen' : 127,
		'Japón' : 128,
		'Jersey' : 129,
		'Jordania' : 130,
		'Kazajistán' : 131,
		'Kenia' : 132,
		'Kirguizistán' : 133,
		'Kiribati' : 134,
		'Kuwait' : 135,
		'Laos' : 136,
		'Lesoto' : 137,
		'Letonia' : 138,
		'Líbano' : 139,
		'Liberia' : 140,
		'Libia' : 141,
		'Liechtenstein' : 142,
		'Lituania' : 143,
		'Luxemburgo' : 144,
		'Macao' : 145,
		'Macedonia' : 146,
		'Madagascar' : 147,
		'Malasia' : 148,
		'Malaui' : 149,
		'Maldivas' : 150,
		'Malí' : 151,
		'Malta' : 152,
		'Man, Isle of' : 153,
		'Marruecos' : 154,
		'Mauricio' : 155,
		'Mauritania' : 156,
		'Mayotte' : 157,
		'Micronesia' : 158,
		'Moldavia' : 159,
		'Mónaco' : 160,
		'Mongolia' : 161,
		'Montenegro' : 162,
		'Montserrat' : 163,
		'Mozambique' : 164,
		'Mundo' : 165,
		'Namibia' : 166,
		'Nauru' : 167,
		'Navassa Island' : 168,
		'Nepal' : 169,
		'Nicaragua' : 170,
		'Níger' : 171,
		'Nigeria' : 172,
		'Niue' : 173,
		'Noruega' : 174,
		'Nueva Caledonia' : 175,
		'Nueva Zelanda' : 176,
		'Omán' : 177,
		'Pacific Ocean' : 178,
		'Países Bajos' : 179,
		'Pakistán' : 180,
		'Palaos' : 181,
		'Panamá' : 182,
		'Papúa-Nueva Guinea' : 183,
		'Paracel Islands' : 184,
		'Paraguay' : 185,
		'Perú' : 186,
		'Polinesia Francesa' : 187,
		'Polonia' : 188,
		'Portugal' : 189,
		'Puerto Rico' : 190,
		'Qatar' : 191,
		'Reino Unido' : 192,
		'República Centroafricana' : 193,
		'República Checa' : 194,
		'República Democrática del Congo' : 195,
		'República Dominicana' : 196,
		'Ruanda' : 197,
		'Rumania' : 198,
		'Rusia' : 199,
		'Sáhara Occidental' : 200,
		'Samoa' : 201,
		'Samoa Americana' : 202,
		'San Cristóbal y Nieves' : 203,
		'San Marino' : 204,
		'San Pedro y Miquelón' : 205,
		'San Vicente y las Granadinas' : 206,
		'Santa Helena' : 207,
		'Santa Lucía' : 208,
		'Santo Tomé y Príncipe' : 209,
		'Senegal' : 210,
		'Serbia' : 211,
		'Seychelles' : 212,
		'Sierra Leona' : 213,
		'Singapur' : 214,
		'Siria' : 215,
		'Somalia' : 216,
		'Southern Ocean' : 217,
		'Spratly Islands' : 218,
		'Sri Lanka' : 219,
		'Suazilandia' : 220,
		'Sudáfrica' : 221,
		'Sudán' : 222,
		'Suecia' : 223,
		'Suiza' : 224,
		'Surinam' : 225,
		'Svalbard y Jan Mayen' : 226,
		'Tailandia' : 227,
		'Taiwán' : 228,
		'Tanzania' : 229,
		'Tayikistán' : 230,
		'Territorio Británico del Océano Indico' : 231,
		'Territorios Australes Franceses' : 232,
		'Timor Oriental' : 233,
		'Togo' : 234,
		'Tokelau' : 235,
		'Tonga' : 236,
		'Trinidad y Tobago' : 237,
		'Túnez' : 238,
		'Turkmenistán' : 239,
		'Turquía' : 240,
		'Tuvalu' : 241,
		'Ucrania' : 242,
		'Uganda' : 243,
		'Unión Europea' : 244,
		'Uruguay' : 245,
		'Uzbekistán' : 246,
		'Vanuatu' : 247,
		'Venezuela' : 248,
		'Vietnam' : 249,
		'Wake Island' : 250,
		'Wallis y Futuna' : 251,
		'West Bank' : 252,
		'Yemen' : 253,
		'Yibuti' : 254,
		'Zambia' : 255
	}

})();

	/**
	 Ejemplo de uso.
	var win = Ti.UI.createWindow({
		backgroundColor:'#FFF'
	});

	var picker = spinner.xmas.get({
		window: win,
		properties:{
			selectionIndicator:true
		},
		data:{
			ano:{
				'1921' : 1921,
				'1981' : 1981
			},
			dos:{
				'1921' : 1921,
				'1981' : 1981
			}
		},
		selected:{
			ano:'1981'
		}
	})

	win.add(picker.element);
	win.open();

	return;
	*/

/*


*/